//
// Created by illyum on 8/15/2024.
//


#include "dmx.h"
#include "serial_port_factory.h"
#include <iostream>
#include <thread>

DMXEngine::DMXEngine(const char* portName)
        : running(false), buffer(DMX_CHANNELS, 0) {
    serialPort = SerialPortFactory::createSerialPort();
    if (!serialPort->open(portName, DMX_BAUD_RATE)) {
        std::cerr << "Failed to open serial port" << std::endl;
        exit(EXIT_FAILURE);
    }
}

DMXEngine::~DMXEngine() {
    if (running) {
        stop();
    }
    delete serialPort;
}

void DMXEngine::start() {
    running = true;
    dmxThread = std::thread(dmxThreadFunc, this);
}

void DMXEngine::stop() {
    running = false;
    if (dmxThread.joinable()) {
        dmxThread.join();
    }

    DMXEngine::dumpZeros();
}

void DMXEngine::setChannelValue(int channel, unsigned char value) {
    if (channel >= 0 && channel < DMX_CHANNELS) {
        buffer[channel] = value;
    }
}

void DMXEngine::sendDMXData() {
    while (running) {
        serialPort->sendBreak();
        std::this_thread::sleep_for(std::chrono::milliseconds(1)); // MAB

        std::vector<unsigned char> data = {0}; // Start code
        data.insert(data.end(), buffer.begin(), buffer.end());

        serialPort->writeData(data);
        std::this_thread::sleep_for(std::chrono::milliseconds(25));
    }
}

void DMXEngine::dumpZeros() {
    std::fill(buffer.begin(), buffer.end(), 0);

    serialPort->sendBreak();
    std::this_thread::sleep_for(std::chrono::milliseconds(1)); // MAB

    std::vector<unsigned char> data = {0}; // Start code
    data.insert(data.end(), buffer.begin(), buffer.end());

    serialPort->writeData(data);
}

void DMXEngine::dmxThreadFunc(DMXEngine* engine) {
    engine->sendDMXData();
}

