//
// Created by illyum on 8/16/2024.

#ifdef _WIN32
#include "serial_port.h"
#include <windows.h>
#include <iostream>
#include <thread>

class WindowsSerialPort : public SerialPort {
public:
    WindowsSerialPort() : hSerial(INVALID_HANDLE_VALUE) {}

    ~WindowsSerialPort() {
        close();
    }

    bool open(const std::string& portName, int baudRate) override {
        hSerial = CreateFileA(portName.c_str(), GENERIC_WRITE, 0, nullptr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr);
        if (hSerial == INVALID_HANDLE_VALUE) {
            std::cerr << "Error opening " << portName << " port" << std::endl;
            return false;
        }
        return configure(baudRate, 8, TWOSTOPBITS, NOPARITY);
    }

    void close() override {
        if (hSerial != INVALID_HANDLE_VALUE) {
            CloseHandle(hSerial);
            hSerial = INVALID_HANDLE_VALUE;
        }
    }

    bool configure(int baudRate, int dataBits, int stopBits, int parity) override {
        DCB dcbSerialParams = { 0 };
        dcbSerialParams.DCBlength = sizeof(dcbSerialParams);
        if (!GetCommState(hSerial, &dcbSerialParams)) {
            std::cerr << "Error getting COM port state" << std::endl;
            return false;
        }

        dcbSerialParams.BaudRate = baudRate;
        dcbSerialParams.ByteSize = dataBits;
        dcbSerialParams.StopBits = stopBits;
        dcbSerialParams.Parity = parity;

        if (!SetCommState(hSerial, &dcbSerialParams)) {
            std::cerr << "Error setting COM port state" << std::endl;
            return false;
        }

        COMMTIMEOUTS timeouts = { 0 };
        timeouts.ReadIntervalTimeout = 50;
        timeouts.ReadTotalTimeoutConstant = 50;
        timeouts.ReadTotalTimeoutMultiplier = 10;
        timeouts.WriteTotalTimeoutConstant = 50;
        timeouts.WriteTotalTimeoutMultiplier = 10;

        return SetCommTimeouts(hSerial, &timeouts);
    }

    bool writeData(const std::vector<unsigned char>& data) override {
        DWORD bytesWritten;
        return WriteFile(hSerial, data.data(), data.size(), &bytesWritten, nullptr);
    }

    bool sendBreak() override {
        EscapeCommFunction(hSerial, SETBREAK);
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
        EscapeCommFunction(hSerial, CLRBREAK);
        std::this_thread::sleep_for(std::chrono::milliseconds(1)); // MAB
        return true;
    }

private:
    HANDLE hSerial;
};
#endif
